/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import javax.servlet.jsp.tagext.TagSupport;


/**
 * <p>
 * This is an abstract implementation of the NodeDiscovery interface and is useful to NodeDiscovery implementations
 * that are also a Tag.  This abstract class simplifies the work required when an implementation of the NodeDiscovery
 * will also provide services as an empty tag and provides an attribute to set the title.  The <b>title</b> attribute
 * (optional) can be null to indicate that the title for the current node should default to that found in the
 * PathDiscovery data source.
 * </p>
 *
 * <p>
 * <b>Implementation notes:</b> The developer should strongly recommend that implementations of NodeDiscovery, which
 * also serve as Tags, inherited from this class. The subclass then does not need to worry about tag rendering. Once
 * the tag is processed, the current node is saved to the BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE attribute using
 * page scope.
 * </p>
 *
 * <p>
 * <b>Thread Safety:</b> This class is thread safe in the context of the JSP rendering process.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public abstract class AbstractNodeDiscovery extends TagSupport implements NodeDiscovery {
    /**
     * <p>
     * Represents the title associated with this node discovery.  This variable is set initially to null, is mutable
     * and can be null but never an empty string.  A null title means that the title for the current node should
     * default to that found in the PathDiscovery data source. This variable is referenced by the associated
     * getter/setter methods.
     * </p>
     */
    private String title = null;

    /**
     * <p>
     * Constructs this abstract base. Simply calls the superclass constructor.
     * </p>
     */
    protected AbstractNodeDiscovery() {
        super();
    }

    /**
     * <p>
     * This method will get the current node and if found, save it to BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE
     * attribute using page scope.
     * </p>
     *
     * @return Always SKIP_BODY to skip the empty body.
     *
     * @throws BreadCrumbException if an exception occurs discovering the node
     */
    public int doStartTag() throws BreadCrumbException {
        // save the current node in the attribute
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE, getCurrentNode(pageContext));

        return SKIP_BODY;
    }

    /**
     * <p>
     * Returns the value of title attribute.
     * </p>
     *
     * @return The possibly null, non-empty string representing the title
     */
    public String getTitle() {
        return title;
    }

    /**
     * <p>
     * Sets the value of title attribute.
     * </p>
     *
     * @param title A possibly null, possibly empty (trim'd) string representing the title
     */
    public void setTitle(String title) {
        this.title = ((title != null) && (title.trim().length() == 0)) ? null : title;
    }

    /**
     * <p>
     * Resets all attribute member variables to their initial values.
     * </p>
     */
    public void release() {
        title = null;
        super.release();
    }
}
