/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import javax.servlet.jsp.PageContext;


/**
 * <p>
 * This interface defines the contract for classes wishing to discover the current node.  Implementations of this
 * interface will be called when the current page has not been defined (i.e. page scope attribute for
 * BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE). The implementation should discover, in it's own way, the current node
 * when getCurrentNode() is called and return the node representation of that url.
 * </p>
 *
 * <p>
 * <b>Implementation notes:</b> Implementations of this class will be created in the constructor of the
 * BreadCrumbTrailTag via the object factory.  The class will be used in the doStartTag() method of the
 * BreadCrumbTrailTag when the page scope attribute (BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE) has not already  been
 * defined.  The node representation of the current url will then be returned by the implementing class and used to
 * discover the path to the root (i.e. a PathDiscovery call).
 * </p>
 *
 * <p>
 * <b>Thread Safety:</b> Implementations should be thread safe in the context of the JSP rendering process.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public interface NodeDiscovery {
    /**
     * <p>
     * Retrieves the node representation of the current page. If the current page could not be found or is not
     * applicable - simply return null. The returned node should include any query string available.
     * </p>
     *
     * @param pageContext A non-null page context to use
     *
     * @return A non-null node representing the current url or null if the node couldn't be determined
     *
     * @throws IllegalArgumentException if the pageContext is null
     * @throws BreadCrumbException if an exception occurs discovering the node
     */
    BreadCrumbNode getCurrentNode(PageContext pageContext)
        throws BreadCrumbException;
}
