/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import javax.servlet.jsp.PageContext;


/**
 * <p>
 * This interface defines the contract for classes wishing to discover the path from current node to the root node.
 * Implementations of this  interface will be called with the current page (specified by the current node) and the
 * current page context. The implementation should discover, in it's own way, the complete path from the current node
 * to the root when getPath() is called.
 * </p>
 *
 * <p>
 * <b>Implementation notes:</b> Implementations of this class will be created in the constructor of the
 * BreadCrumbTrailTag via the object factory.  The class will be used in the doStartTag() method of the
 * BreadCrumbTrailTag when the path should be discovered.  The returned path, if any, will then be used to format the
 * html for the tag (i.e. each node will be passed to the node formatter for formatting with path separators in
 * between).
 * </p>
 *
 * <p>
 * <b>Thread Safety:</b> Implementations should be thread safe in the context of the JSP rendering process
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public interface PathDiscovery {
    /**
     * <p>
     * Returns the path from the current node for the given page context.  The current node can have a null title if
     * the current node should default to that found in the data source for this implementation.  The returned nodes,
     * however, should have a  title assigned to each node.  The current node's url can include query strings on both
     * the current node that is called and in the nodes that are returned in the path.  Method implementations should
     * return an array where the 0th index position is the root node and the nth-1 index position is the current node,
     * or return null if the current path could not be found.
     * </p>
     *
     * @param currentNode A non-null node representing the current page
     * @param pageContext A non-null page context to use
     *
     * @return An array of BreadCrumbNodes representing the path from root (0th index) to current node (nth-1 index),
     *         or null if path couldn't be found
     *
     * @throws IllegalArgumentException if any argument is null
     * @throws BreadCrumbException if an exception occurs discovering the page
     */
    BreadCrumbNode[] getPath(BreadCrumbNode currentNode, PageContext pageContext)
        throws BreadCrumbException;
}
