/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodeformatter;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;

import javax.servlet.jsp.PageContext;


/**
 * A utility class to provide common methods used by the node formatters. This class is package-private so that it can
 * be used by classes in the same package.
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
class FormatterUtil {
    /**
     * Private constructor to prevent outside instantiation.
     */
    private FormatterUtil() {
    }

    /**
     * Gets the next node to format from the BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE attribute.
     *
     * @param pageContext the page context to use.
     *
     * @return the next node to format.
     *
     * @throws BreadCrumbException if the next node is not available.
     */
    public static BreadCrumbNode getFormatNode(PageContext pageContext)
        throws BreadCrumbException {
        Object node = pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE);

        if (!(node instanceof BreadCrumbNode)) {
            throw new BreadCrumbException("Cannot find the next node to format");
        }
        return (BreadCrumbNode) node;
    }

    /**
     * Gets the CSS style class to apply to the next node from the BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS
     * attribute. If it is not available (because the style is not specified), null is returned.
     *
     * @param pageContext the page context to use.
     *
     * @return the CSS style class to apply, can be null.
     *
     * @throws BreadCrumbException if the attribute does not reference a String instance (including null).
     */
    public static String getFormatStyleClass(PageContext pageContext)
        throws BreadCrumbException {
        Object styleClass = pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS);

        if (styleClass != null) {
            if (!(styleClass instanceof String)) {
                throw new BreadCrumbException("Cannot find the style class to apply");
            } else if (((String) styleClass).trim().length() == 0) {
                styleClass = null;  // empty string is translated to null
            }
        }
        return (String) styleClass;
    }
}
