/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;

import java.util.Map;


/**
 * <p>
 * Defines the contract for a data source for the site. Implementations of this inteface should create and return a
 * site map consisting of NodeMatcher instances that describe the site map. The returned map should have each unique
 * NodeMatcher listed as the key and a List of NodeMatcher instances that describe the children of that key.
 * </p>
 *
 * <p>
 * <b>Implementation notes:</b> The site discovery data source will be used to read in the site information and create
 * a map of nodes to child nodes as the site map. Each of these nodes will be of NodeMatcher type to allow the
 * component to determine if a node matches any given node in the site map.
 * </p>
 *
 * <p>
 * <b>Thread Safety:</b> Implementations of this interface should be thread safe.
 * </p>
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 */
public interface SiteDiscovery {
    /**
     * <p>
     * Returns a map of NodeMatcher keys to a List of NodeMatcher elements as the value. The key NodeMatcher represents
     * the parent and the value NodeMatchers represent the child values. The value can be null or an empty list to
     * represent no child elements.
     * </p>
     *
     * @return A non-null map of parent NodeMatcher to a list of NodeMatcher children
     *
     * @throws BreadCrumbException if an exception occurs retrieving the site map
     */
    Map getSiteMap() throws BreadCrumbException;
}
