/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.apache.cactus.JspTestCase;
import org.apache.cactus.WebResponse;

import javax.servlet.jsp.tagext.BodyContent;


/**
 * Unit tests for BreadCrumbTrailTag implementation.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class BreadCrumbTrailTagTest extends JspTestCase {
    /**
     * The BreadCrumbTrailTag instance to test against.
     */
    private BreadCrumbTrailTag tag = null;

    /**
     * The current node to get the trail with.
     */
    private BreadCrumbNode node = null;

    /**
     * The body content of the tag.
     */
    private BodyContent bodyContent = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(BreadCrumbTrailTagTest.class);

        return suite;
    }

    /**
     * Prepares a BreadCrumbTrailTag instance for testing. The associated site map has this structure: a->b->d,
     * a->c->e. The current node matches the node d, and is set to the attribute in page scope.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    protected void setUp() throws Exception {
        ConfigHelper.loadConfig();

        bodyContent = pageContext.pushBody();

        tag = new BreadCrumbTrailTag();
        tag.setPageContext(pageContext);
        tag.setBodyContent(bodyContent);

        node = new BreadCrumbNode("d", "title");
        pageContext.setAttribute(tag.BREADCRUMB_CURRENT_NODE, node);
    }

    /**
     * Clears all the namespaces.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    protected void tearDown() throws Exception {
        pageContext.popBody();
        ConfigHelper.unloadConfig();
    }

    /**
     * Test of constructor. Verifies the attribute values. This proves that the constructor loads the overriding values
     * from the config.
     */
    public void testConstructor() {
        tag = new BreadCrumbTrailTag();
        assertEquals("Fails to set path separator", "?", tag.getPathSeparator());
    }

    /**
     * Test of getPathDirection and setPathDirection methods with valid path direction. Verifies it by getting the path
     * direction back after setting it.
     */
    public void testGetSetPathDirection_Valid() {
        tag.setPathDirection(tag.CURRENT_TO_ROOT);
        assertEquals("Returns an incorrect path direction", tag.CURRENT_TO_ROOT, tag.getPathDirection());
    }

    /**
     * Test of setPathDirection method with invalid path direction. Expects IllegalArgumentException.
     */
    public void testSetPathDirection_Invalid() {
        try {
            tag.setPathDirection(-1);
            fail("Sets invalid path direction");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getPathSeparator and setPathSeparator methods with non-null path separator. Verifies it by getting the
     * path separator back after setting it.
     */
    public void testGetSetPathSeparator_NonNull() {
        tag.setPathSeparator("?");
        assertEquals("Returns an incorrect path separator", "?", tag.getPathSeparator());
    }

    /**
     * Test of setPathSeparator method with null path separator. Expects IllegalArgumentException.
     */
    public void testSetPathSeparator_Null() {
        try {
            tag.setPathSeparator(null);
            fail("Sets null path separator");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getPathSeparatorStyle and setPathSeparatorStyle methods. Verifies it by getting the path separator style
     * back after setting it.
     */
    public void testGetSetPathSeparatorStyle() {
        tag.setPathSeparatorStyle("style");
        assertEquals("Returns an incorrect path separator style", "style", tag.getPathSeparatorStyle());
    }

    /**
     * Test of getNodeStyle and setNodeStyle methods. Verifies it by getting the node style back after setting it.
     */
    public void testGetSetNodeStyle() {
        tag.setNodeStyle("style");
        assertEquals("Returns an incorrect node style", "style", tag.getNodeStyle());
    }

    /**
     * Test of getCurrentNodeStyle and setCurrentNodeStyle methods. Verifies it by getting the current node style back
     * after setting it.
     */
    public void testGetSetCurrentNodeStyle() {
        tag.setCurrentNodeStyle("style");
        assertEquals("Returns an incorrect current node style", "style", tag.getCurrentNodeStyle());
    }

    /**
     * Test of getRootNodeStyle and setRootNodeStyle methods. Verifies it by getting the root node style back after
     * setting it.
     */
    public void testGetSetRootNodeStyle() {
        tag.setRootNodeStyle("style");
        assertEquals("Returns an incorrect root node style", "style", tag.getRootNodeStyle());
    }

    /**
     * Test of getMouseOverStyle and setMouseOverStyle methods. Verifies it by getting the mouse over style back after
     * setting it.
     */
    public void testGetSetMouseOverStyle() {
        tag.setMouseOverStyle("style");
        assertEquals("Returns an incorrect mouse over style", "style", tag.getMouseOverStyle());
    }

    /**
     * Test of getPathOverride and setPathOverride methods. Verifies it by getting the path override back after setting
     * it.
     */
    public void testGetSetPathOverride() {
        tag.setPathOverride("pathOverride");
        assertEquals("Returns an incorrect path override", "pathOverride", tag.getPathOverride());
    }

    /**
     * Test of getSilentErrors and setSilentErrors methods. Verifies it by getting the silent errors back after setting
     * it.
     */
    public void testGetSetSilentErrors() {
        tag.setSilentErrors(true);
        assertTrue("Returns an incorrect silent errors flag", tag.getSilentErrors());
    }

    /**
     * Test of getTitle and setTitle methods. Verifies it by getting the title back after setting it.
     */
    public void testGetSetTitle() {
        tag.setTitle("title");
        assertEquals("Returns an incorrect path override", "title", tag.getTitle());
    }

    /**
     * Test of doStartTag method with false silentErrors. Expects BreadCrumbException when any error occurs.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoStartTag_FalseSilentErrors() throws Exception {
        // make sure some error will happen
        ConfigHelper.unloadConfig();

        try {
            tag.doStartTag();
            fail("Error is ignored when silentErrors is false");
        } catch (BreadCrumbException e) {
            // good
        }
    }

    /**
     * Test of doStartTag method with true silentErrors. Should return SKIP_BODY when any error occurs.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoStartTag_TrueSilentErrors() throws Exception {
        // make sure some error will happen
        ConfigHelper.unloadConfig();

        tag.setSilentErrors(true);

        assertEquals("SKIP_BODY is not returned", tag.SKIP_BODY, tag.doStartTag());
    }

    /**
     * Test of doStartTag method with BREADCRUMB_CURRENT_NODE attribute set to the current node. The current node
     * matches d in the site map used by the XmlPathDiscovery. Should return EVAL_BODY_BUFFERED.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoStartTag_CurrentNodeAttr() throws Exception {
        assertEquals("Fails to find current node from BREADCRUMB_CURRENT_NODE attribute", tag.EVAL_BODY_BUFFERED,
            tag.doStartTag());
    }

    /**
     * Test of doStartTag method with no BREADCRUMB_CURRENT_NODE attribute set to the current node. The current node
     * does not match any node in the site map used by the XmlPathDiscovery, since its url is found by the
     * HttpServletNodeDiscoveryTag. Should return SKIP_BODY. This proves that the tag uses an instance of
     * HttpServletNodeDiscoveryTag from the object factory.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoStartTag_NoCurrentNodeAttr() throws Exception {
        // make sure the current node attribute is not set
        pageContext.removeAttribute(tag.BREADCRUMB_CURRENT_NODE);

        assertEquals("Fails to find current node from the specified node discovery", tag.SKIP_BODY, tag.doStartTag());
    }

    /**
     * Test of doStartTag method with pathOverride attribute set to the path a->d. The current node matches d in the
     * site map. Verifies if the body iterates for 2 times. This proves that the tag constructs the site map from the
     * path override.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoStartTag_PathOverride() throws Exception {
        tag.setPathOverride("<BreadCrumb><bc_node title=\"a\" url=\"/\" pattern=\"a\">"
            + "<bc_node title=\"d\" url=\"/\" pattern=\"d\"/></bc_node></BreadCrumb>");

        // start the tag and enter the body
        tag.doStartTag();
        tag.doInitBody();

        int count = 0;
        int result;

        do {
            count++;
            result = tag.doAfterBody();
        } while (result == tag.EVAL_BODY_AGAIN);

        assertEquals("Iterates for an incorrect number of times", 2, count);
        assertEquals("SKIP_BODY is not returned after iterations are done", tag.SKIP_BODY, result);
    }

    /**
     * Test of doStartTag method with no pathOverride attribute set. The current node matches d in the site map used by
     * the XmlPathDiscovery whose path is a->b->d. Verifies if the body iterates for 3 times. This proves that the tag
     * uses an instance of XmlPathDiscovery from the object factory.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoStartTag_NoPathOverride() throws Exception {
        // start the tag and enter the body
        tag.doStartTag();
        tag.doInitBody();

        int count = 0;
        int result;

        do {
            count++;
            result = tag.doAfterBody();
        } while (result == tag.EVAL_BODY_AGAIN);

        assertEquals("Iterates for an incorrect number of times", 3, count);
        assertEquals("SKIP_BODY is not returned after iterations are done", tag.SKIP_BODY, result);
    }

    /**
     * Test of doStartTag method with no style specified.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoStartTag_NoStyles() throws Exception {
        tag.doStartTag();
    }

    /**
     * Verifies if no output string is written.
     *
     * @param theResponse the response object to check the output string.
     */
    public void endDoStartTag_NoStyles(WebResponse theResponse) {
        assertEquals("Prints an incorrect output", 0, theResponse.getText().length());
    }

    /**
     * Test of doStartTag method with some styles specified.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoStartTag_Styles() throws Exception {
        pageContext.popBody();
        tag.setPathSeparatorStyle("style1");
        tag.setNodeStyle("style2");
        tag.setCurrentNodeStyle("style3");
        tag.setRootNodeStyle("style4");
        tag.setMouseOverStyle("style5");
        tag.doStartTag();
    }

    /**
     * Verifies if the span tag specifying the CSS styles is written.
     *
     * @param theResponse the response object to check the output string.
     */
    public void endDoStartTag_Styles(WebResponse theResponse) {
        String[] lines = theResponse.getTextAsArray();

        assertEquals("Prints an incorrect output", "<span>", lines[0]);
        assertEquals("Prints an incorrect output", "<style>", lines[1]);
        assertEquals("Prints an incorrect output", "path-separator-style {style1}", lines[2]);
        assertEquals("Prints an incorrect output", ".node-style a:link {style2}", lines[3]);
        assertEquals("Prints an incorrect output", ".current-node-style a:link {style3}", lines[4]);
        assertEquals("Prints an incorrect output", ".root-node-style a:link {style4}", lines[5]);
        assertEquals("Prints an incorrect output", ".node-style a:hover {style2}", lines[6]);
        assertEquals("Prints an incorrect output", ".current-node-style a:hover {style3}", lines[7]);
        assertEquals("Prints an incorrect output", ".root-node-style a:hover {style4}", lines[8]);
        assertEquals("Prints an incorrect output", "</style>", lines[9]);
    }

    /**
     * Test of doInitBody method with path direction set to ROOT_TO_CURRENT and no root node style specified. The trail
     * should be a->b->d from the site map. Verifies if the BREADCRUMB_FORMAT_NODE attribute is set to the node a.
     * Also verifies if the BREADCRUMB_FORMAT_STYLE_CLASS attribute is not set after calling it.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoInitBody_RootToCurrent_NoRootNodeStyle()
        throws Exception {
        tag.doStartTag();
        tag.doInitBody();

        BreadCrumbNode node = (BreadCrumbNode) pageContext.getAttribute(tag.BREADCRUMB_FORMAT_NODE);
        String styleClass = (String) pageContext.getAttribute(tag.BREADCRUMB_FORMAT_STYLE_CLASS);

        assertNotNull("Fails to set BREADCRUMB_FORMAT_NODE attribute", node);
        assertEquals("Fails to go from root to current", "a", node.getUrl());
        assertNull("Fails to set BREADCRUMB_FORMAT_STYLE_CLASS attribute", styleClass);
    }

    /**
     * Test of doInitBody method with path direction set to ROOT_TO_CURRENT and root node style specified. The trail
     * should be a->b->d from the site map. Verifies if the BREADCRUMB_FORMAT_NODE attribute is set to the node a.
     * Also verifies if the BREADCRUMB_FORMAT_STYLE_CLASS attribute is set after calling it.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoInitBody_RootToCurrent_RootNodeStyle()
        throws Exception {
        tag.setRootNodeStyle("style");

        tag.doStartTag();
        tag.doInitBody();

        BreadCrumbNode node = (BreadCrumbNode) pageContext.getAttribute(tag.BREADCRUMB_FORMAT_NODE);
        String styleClass = (String) pageContext.getAttribute(tag.BREADCRUMB_FORMAT_STYLE_CLASS);

        assertNotNull("Fails to set BREADCRUMB_FORMAT_NODE attribute", node);
        assertEquals("Fails to go from root to current", "a", node.getUrl());
        assertEquals("Fails to set BREADCRUMB_FORMAT_STYLE_CLASS attribute", "root-node-style", styleClass);
    }

    /**
     * Test of doInitBody method with path direction set to CURRENT_TO_ROOT and no current node style specified. The
     * trail should be d->b->a from the site map. Verifies if the BREADCRUMB_FORMAT_NODE attribute is set to the node
     * d. Also verifies if the BREADCRUMB_FORMAT_STYLE_CLASS attribute is not set after calling it.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoInitBody_CurrentToRoot_NoCurrentNodeStyle()
        throws Exception {
        tag.setPathDirection(tag.CURRENT_TO_ROOT);

        tag.doStartTag();
        tag.doInitBody();

        BreadCrumbNode node = (BreadCrumbNode) pageContext.getAttribute(tag.BREADCRUMB_FORMAT_NODE);
        String styleClass = (String) pageContext.getAttribute(tag.BREADCRUMB_FORMAT_STYLE_CLASS);

        assertNotNull("Fails to set BREADCRUMB_FORMAT_NODE attribute", node);
        assertEquals("Fails to go from root to current", "d", node.getUrl());
        assertNull("Fails to set BREADCRUMB_FORMAT_STYLE_CLASS attribute", styleClass);
    }

    /**
     * Test of doInitBody method with path direction set to CURRENT_TO_ROOT and current node style specified. The trail
     * should be d->b->a from the site map. Verifies if the BREADCRUMB_FORMAT_NODE attribute is set to the node d.
     * Also verifies if the BREADCRUMB_FORMAT_STYLE_CLASS attribute is set after calling it.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoInitBody_CurrentToRoot_CurrentNodeStyle()
        throws Exception {
        tag.setPathDirection(tag.CURRENT_TO_ROOT);
        tag.setCurrentNodeStyle("style");

        tag.doStartTag();
        tag.doInitBody();

        BreadCrumbNode node = (BreadCrumbNode) pageContext.getAttribute(tag.BREADCRUMB_FORMAT_NODE);
        String styleClass = (String) pageContext.getAttribute(tag.BREADCRUMB_FORMAT_STYLE_CLASS);

        assertNotNull("Fails to set BREADCRUMB_FORMAT_NODE attribute", node);
        assertEquals("Fails to go from root to current", "d", node.getUrl());
        assertEquals("Fails to set BREADCRUMB_FORMAT_STYLE_CLASS attribute", "current-node-style", styleClass);
    }

    /**
     * Test of doAfterBody method with path separator style not specified. Inserts some body content. Iterates until it
     * returns SKIP_BODY.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoAfterBody_NoPathSeparatorStyle()
        throws Exception {
        tag.doStartTag();
        tag.doInitBody();

        int result;

        do {
            bodyContent.print("body");
            result = tag.doAfterBody();
        } while (result == tag.EVAL_BODY_AGAIN);
    }

    /**
     * Verifies if the each body is written with the path separator in between. No span tag is used for the path
     * separator style.
     *
     * @param theResponse the response object to check the output string.
     */
    public void endDoAfterBody_NoPathSeparatorStyle(WebResponse theResponse) {
        assertEquals("Prints an incorrect output", "body?body?body", theResponse.getText());
    }

    /**
     * Test of doAfterBody method with path separator style specified. Inserts some body content. Iterates until it
     * returns SKIP_BODY.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoAfterBody_PathSeparatorStyle() throws Exception {
        tag.setPathSeparatorStyle("style");
        tag.doStartTag();
        tag.doInitBody();

        int result;

        do {
            bodyContent.print("body");
            result = tag.doAfterBody();
        } while (result == tag.EVAL_BODY_AGAIN);
    }

    /**
     * Verifies if the each body is written with the path separator in between. The span tag is used for the path
     * separator style.
     *
     * @param theResponse the response object to check the output string.
     */
    public void endDoAfterBody_PathSeparatorStyle(WebResponse theResponse) {
        String expected = "body<span class=\"path-separator-style\">?</span>body<span class=\"path-separator-style\">?</span>body";

        assertTrue("Prints an incorrect output", theResponse.getText().indexOf(expected) >= 0);
    }

    /**
     * Test of doEndTag method with no style specified. Should return EVAL_PAGE.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoEndTag_NoStyles() throws Exception {
        assertEquals("EVAL_PAGE is not returned", tag.EVAL_PAGE, tag.doEndTag());
    }

    /**
     * Verifies if no output string is written.
     *
     * @param theResponse the response object to check the output string.
     */
    public void endDoEndTag_NoStyles(WebResponse theResponse) {
        assertEquals("Prints an incorrect output", 0, theResponse.getText().length());
    }

    /**
     * Test of doEndTag method with some styles specified.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoEndTag_Styles() throws Exception {
        tag.setPathSeparatorStyle("style1");
        assertEquals("EVAL_PAGE is not returned", tag.EVAL_PAGE, tag.doEndTag());
    }

    /**
     * Verifies if the closing span tag is written to the output.
     *
     * @param theResponse the response object to check the output string.
     */
    public void endDoEndTag_Styles(WebResponse theResponse) {
        assertEquals("Prints an incorrect output", "</span>", theResponse.getText().trim());
    }

    /**
     * Test of release method. Verifies if data members are reset to their initial config values.
     */
    public void testRelease() {
        // set before release
        tag.setPathSeparator(":");
        tag.setPathSeparatorStyle("style");
        tag.setNodeStyle("style");
        tag.setCurrentNodeStyle("style");
        tag.setRootNodeStyle("style");
        tag.setMouseOverStyle("style");
        tag.setSilentErrors(true);
        tag.setPathOverride("pathOverride");
        tag.setTitle("title");

        tag.release();

        // verify after release
        assertEquals("Fails to reset path separator", "?", tag.getPathSeparator());
        assertNull("Fails to reset path separator style", tag.getPathSeparatorStyle());
        assertNull("Fails to reset node style", tag.getNodeStyle());
        assertNull("Fails to reset current node style", tag.getCurrentNodeStyle());
        assertNull("Fails to reset root node style", tag.getRootNodeStyle());
        assertNull("Fails to reset mouse over style", tag.getMouseOverStyle());
        assertFalse("Fails to reset silent errors", tag.getSilentErrors());
        assertNull("Fails to reset path override", tag.getPathOverride());
        assertNull("Fails to reset title", tag.getTitle());
    }
}
