/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodediscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.apache.cactus.JspTestCase;
import org.apache.cactus.WebRequest;


/**
 * Unit tests for HttpServletNodeDiscoveryTag implementation.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class HttpServletNodeDiscoveryTagTest extends JspTestCase {
    /**
     * The title of the HttpServletNodeDiscoveryTag.
     */
    private static final String TITLE = "title";

    /**
     * The HttpServletNodeDiscoveryTag instance to test against.
     */
    private HttpServletNodeDiscoveryTag tag = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(HttpServletNodeDiscoveryTagTest.class);

        return suite;
    }

    /**
     * Prepares an HttpServletNodeDiscoveryTag instance for testing.
     */
    protected void setUp() {
        tag = new HttpServletNodeDiscoveryTag();
        tag.setTitle("title");
        tag.setPageContext(pageContext);
    }

    /**
     * Tests accessibility of constructor.
     */
    public void testConstructor() {
        new HttpServletNodeDiscoveryTag();
    }

    /**
     * Test of getCurrentNode method with null page context. Expects IllegalArgumentException.
     */
    public void testGetCurrentNode_NullPageContext() {
        try {
            tag.getCurrentNode(null);
            fail("Gets current node with null page context");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getCurrentNode method on uri. Verifies if the url of the returned node starts with the request uri.
     */
    public void testGetCurrentNode_OnUri() {
        BreadCrumbNode node = tag.getCurrentNode(pageContext);

        assertTrue("Fails to set the uri", node.getUrl().startsWith(request.getRequestURI()));
    }

    /**
     * Passes some query string to the request parameters.
     *
     * @param theRequest the request object on the client side.
     */
    public void beginGetCurrentNode_OnQueryString(WebRequest theRequest) {
        theRequest.addParameter("name", "a b c");
    }

    /**
     * Test of getCurrentNode method on query string. Verifies if the url of the returned node contains the decoded
     * query string.
     */
    public void testGetCurrentNode_OnQueryString() {
        BreadCrumbNode node = tag.getCurrentNode(pageContext);

        assertTrue("Fails to set the uri", node.getUrl().startsWith(request.getRequestURI() + "?"));
        assertTrue("Fails to set the query string", node.getUrl().indexOf("name=a b c") >= 0);
    }

    /**
     * Test of getCurrentNode method on title. Verifies if the returned node has the title as the title attribute.
     */
    public void testGetCurrentNode_OnTitle() {
        BreadCrumbNode node = tag.getCurrentNode(pageContext);

        assertEquals("Fails to set the title", TITLE, node.getTitle());
    }
}
