/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodediscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.apache.cactus.JspTestCase;

import javax.servlet.jsp.tagext.BodyContent;


/**
 * Unit tests for InlineTextNodeDiscoveryTag implementation.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class InlineTextNodeDiscoveryTagTest extends JspTestCase {
    /**
     * The title of the InlineTextNodeDiscoveryTag.
     */
    private static final String TITLE = "title";

    /**
     * The InlineTextNodeDiscoveryTag instance to test against.
     */
    private InlineTextNodeDiscoveryTag tag = null;

    /**
     * The body content of the tag.
     */
    private BodyContent bodyContent = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(InlineTextNodeDiscoveryTagTest.class);

        return suite;
    }

    /**
     * Prepares an InlineTextNodeDiscoveryTag instance for testing.
     */
    protected void setUp() {
        bodyContent = pageContext.pushBody();

        tag = new InlineTextNodeDiscoveryTag();
        tag.setTitle("title");
        tag.setPageContext(pageContext);
        tag.setBodyContent(bodyContent);
    }

    /**
     * Pops the body content.
     */
    protected void tearDown() {
        pageContext.popBody();
    }

    /**
     * Tests accessibility of constructor.
     */
    public void testConstructor() {
        new InlineTextNodeDiscoveryTag();
    }

    /**
     * Test of doStartTag method. Should return EVAL_BODY_BUFFERED.
     */
    public void testDoStartTag() {
        assertEquals("EVAL_BODY_BUFFERED is not returned", tag.EVAL_BODY_BUFFERED, tag.doStartTag());
    }

    /**
     * Test of doAfterBody method with empty body content. Expects BreadCrumbException.
     */
    public void testDoAfterBody_EmptyBodyContent() {
        try {
            tag.doAfterBody();
            fail("Does body processing with empty body content");
        } catch (BreadCrumbException e) {
            // good
        }
    }

    /**
     * Test of doAfterBody method with non-empty body content. Should return SKIP_BODY. Also verifies if the
     * BREADCRUMB_CURRENT_NODE attribute references a non-null node. Verifies if the body content is set as the node's
     * url. The url should be decoded. Also verifies if the node has the title as the title attribute.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoAfterBody_NonEmptyBodyContent() throws Exception {
        bodyContent.print("/test/test.jsp?name=a+b+c");

        assertEquals("SKIP_BODY is not returned", tag.SKIP_BODY, tag.doAfterBody());

        BreadCrumbNode node = (BreadCrumbNode) pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE);

        assertNotNull("Fails to set current node to attribute", node);
        assertEquals("Fails to set the url", "/test/test.jsp?name=a b c", node.getUrl());
        assertEquals("Fails to set the title", TITLE, node.getTitle());
    }

    /**
     * Test of getTitle and setTitle methods. Verifies it by getting the title back after setting it.
     */
    public void testGetSetTitle() {
        String title = "title2";

        tag.setTitle(title);
        assertEquals("Returns an incorrect title", title, tag.getTitle());
    }

    /**
     * Test of release method. Verifies if data members are reset to their initial values.
     */
    public void testRelease() {
        tag.release();
        assertNull("Fails to reset title", tag.getTitle());
    }
}
