/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodeformatter;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbTrailTag;

import junit.framework.Test;
import junit.framework.TestSuite;

import org.apache.cactus.JspTestCase;
import org.apache.cactus.WebResponse;


/**
 * Unit tests for HtmlAnchorNodeFormatterTag implementation.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class HtmlAnchorNodeFormatterTagTest extends JspTestCase {
    /**
     * The HtmlAnchorNodeFormatterTag instance to test against.
     */
    private HtmlAnchorNodeFormatterTag tag = null;

    /**
     * The node set to the BREADCRUMB_FORMAT_NODE attribute.
     */
    private BreadCrumbNode node = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(HtmlAnchorNodeFormatterTagTest.class);

        return suite;
    }

    /**
     * Prepares an HtmlAnchorNodeFormatterTag instance for testing. Also sets the BREADCRUMB_FORMAT_NODE attribute in
     * page scope.
     */
    protected void setUp() {
        tag = new HtmlAnchorNodeFormatterTag();
        tag.setPageContext(pageContext);

        node = new BreadCrumbNode("url", "title");
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, node);
    }

    /**
     * Tests accessibility of constructor.
     */
    public void testConstructor() {
        new HtmlAnchorNodeFormatterTag();
    }

    /**
     * Test of doStartTag method with no BREADCRUMB_FORMAT_NODE attribute. Expects BreadCrumbException.
     */
    public void testDoStartTag_NoNodeAttr() {
        // make sure the attribute is not set
        pageContext.removeAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE);

        try {
            tag.doStartTag();
            fail("Does start tag processing with no BREADCRUMB_FORMAT_NODE attribute");
        } catch (BreadCrumbException e) {
            // good
        }
    }

    /**
     * Test of doStartTag method with bad BREADCRUMB_FORMAT_NODE attribute (non-BreadCrumbNode type). Expects
     * BreadCrumbException.
     */
    public void testDoStartTag_BadNodeAttr() {
        // make sure the attribute is bad
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, "string");

        try {
            tag.doStartTag();
            fail("Does start tag processing with bad BREADCRUMB_FORMAT_NODE attribute");
        } catch (BreadCrumbException e) {
            // good
        }
    }

    /**
     * Test of doStartTag method with bad BREADCRUMB_FORMAT_STYLE_CLASS attribute (non-String type). Expects
     * BreadCrumbException.
     */
    public void testDoStartTag_BadStyleClassAttr() {
        // make sure the attribute is bad
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, new Integer(0));

        try {
            tag.doStartTag();
            fail("Does start tag processing with bad BREADCRUMB_FORMAT_STYLE_CLASS attribute");
        } catch (BreadCrumbException e) {
            // good
        }
    }

    /**
     * Test of doStartTag method with title specified in the node. Should return SKIP_BODY.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoStartTag_WithTitle() throws Exception {
        assertEquals("SKIP_BODY is not returned", tag.SKIP_BODY, tag.doStartTag());
    }

    /**
     * Verifies if the html anchor is written with the title as specified.
     *
     * @param theResponse the response object to check the output string.
     */
    public void endDoStartTag_WithTitle(WebResponse theResponse) {
        assertEquals("Prints an incorrect output", "<a href=\"url\">title</a>", theResponse.getText());
    }

    /**
     * Test of doStartTag method with no title specified in the node. Should return SKIP_BODY.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoStartTag_WithoutTitle() throws Exception {
        // make sure the node has no title
        node = new BreadCrumbNode("url", null);
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_NODE, node);

        assertEquals("SKIP_BODY is not returned", tag.SKIP_BODY, tag.doStartTag());
    }

    /**
     * Verifies if the html anchor is written with the title as the string "Unspecified".
     *
     * @param theResponse the response object to check the output string.
     */
    public void endDoStartTag_WithoutTitle(WebResponse theResponse) {
        assertEquals("Prints an incorrect output", "<a href=\"url\">Unspecified</a>", theResponse.getText());
    }

    /**
     * Test of doStartTag method with style class specified in the attribute. Should return SKIP_BODY.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testDoStartTag_WithStyleClass() throws Exception {
        // make sure the style class is set
        pageContext.setAttribute(BreadCrumbTrailTag.BREADCRUMB_FORMAT_STYLE_CLASS, "styleClass");

        assertEquals("SKIP_BODY is not returned", tag.SKIP_BODY, tag.doStartTag());
    }

    /**
     * Verifies if the html anchor is written with the style class attribute.
     *
     * @param theResponse the response object to check the output string.
     */
    public void endDoStartTag_WithStyleClass(WebResponse theResponse) {
        assertEquals("Prints an incorrect output", "<a href=\"url\" class=\"styleClass\">title</a>",
            theResponse.getText());
    }
}
