/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;


/**
 * Unit tests for AbstractNodeMatcher implementation.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class AbstractNodeMatcherTest extends TestCase {
    /**
     * The title of the AbstractNodeMatcher.
     */
    private static final String TITLE = "title";

    /**
     * The url of the AbstractNodeMatcher.
     */
    private static final String URL = "url";

    /**
     * The AbstractNodeMatcher instance to test against.
     */
    private AbstractNodeMatcher nodeMatcher = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(AbstractNodeMatcherTest.class);

        return suite;
    }

    /**
     * Prepares an AbstractNodeMatcher instance for testing.
     */
    protected void setUp() {
        nodeMatcher = new SimpleAbstractNodeMatcher(TITLE, URL);
    }

    /**
     * Test of constructor with null title. Expects IllegalArgumentException.
     */
    public void testConstructor_NullTitle() {
        try {
            new SimpleAbstractNodeMatcher(null, URL);
            fail("Creates AbstractNodeMatcher with null title");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty title. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyTitle() {
        try {
            new SimpleAbstractNodeMatcher("", URL);
            fail("Creates AbstractNodeMatcher with empty title");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with null url. Expects IllegalArgumentException.
     */
    public void testConstructor_NullUrl() {
        try {
            new SimpleAbstractNodeMatcher(TITLE, null);
            fail("Creates AbstractNodeMatcher with null url");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty url. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyUrl() {
        try {
            new SimpleAbstractNodeMatcher(TITLE, "");
            fail("Creates AbstractNodeMatcher with empty url");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getTitle method. Verifies if it returns the title set in constructor.
     */
    public void testGetTitle() {
        assertEquals("Returns an incorrect title", TITLE, nodeMatcher.getTitle());
    }

    /**
     * Test of getUrl method. Verifies if it returns the url set in constructor.
     */
    public void testGetUrl() {
        assertEquals("Returns an incorrect url", URL, nodeMatcher.getUrl());
    }

    /**
     * A simple implementation of AbstractNodeMatcher class for testing.
     */
    private class SimpleAbstractNodeMatcher extends AbstractNodeMatcher {
        /**
         * Constructor from the given title and url.
         *
         * @param title A non-null, non-empty (trim'd) title to use
         * @param url A non-null, non-empty (trim'd) url to use
         *
         * @throws IllegalArgumentException if any argument is null or an empty (trim'd) string
         */
        public SimpleAbstractNodeMatcher(String title, String url) {
            super(title, url);
        }

        /**
         * Determines whether the specified node matches this node matcher.
         *
         * @param node A non-null node to test
         *
         * @return always false.
         */
        public boolean matches(BreadCrumbNode node) {
            return false;
        }
    }
}
