/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.nodematchers;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbNode;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;


/**
 * Unit tests for RegexNodeMatcher implementation.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 */
public class RegexNodeMatcherTest extends TestCase {
    /**
     * The title of the RegexNodeMatcher.
     */
    private static final String TITLE = "title";

    /**
     * The url of the RegexNodeMatcher.
     */
    private static final String URL = "url";

    /**
     * The node pattern of the RegexNodeMatcher.
     */
    private static final String NODE_PATTERN = "123";

    /**
     * The RegexNodeMatcher instance to test against.
     */
    private RegexNodeMatcher nodeMatcher = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(RegexNodeMatcherTest.class);

        return suite;
    }

    /**
     * Prepares a RegexNodeMatcher instance for testing.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    protected void setUp() throws Exception {
        nodeMatcher = new RegexNodeMatcher(TITLE, URL, NODE_PATTERN);
    }

    /**
     * Test of constructor with null title. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_NullTitle() throws Exception {
        try {
            new RegexNodeMatcher(null, URL, NODE_PATTERN);
            fail("Creates RegexNodeMatcher with null title");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty title. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_EmptyTitle() throws Exception {
        try {
            new RegexNodeMatcher("", URL, NODE_PATTERN);
            fail("Creates RegexNodeMatcher with empty title");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with null url. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_NullUrl() throws Exception {
        try {
            new RegexNodeMatcher(TITLE, null, NODE_PATTERN);
            fail("Creates RegexNodeMatcher with null url");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty url. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_EmptyUrl() throws Exception {
        try {
            new RegexNodeMatcher(TITLE, "", NODE_PATTERN);
            fail("Creates RegexNodeMatcher with empty url");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with null node pattern. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_NullNodePattern() throws Exception {
        try {
            new RegexNodeMatcher(TITLE, URL, null);
            fail("Creates RegexNodeMatcher with null node pattern");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty node pattern. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_EmptyNodePattern() throws Exception {
        try {
            new RegexNodeMatcher(TITLE, URL, "");
            fail("Creates RegexNodeMatcher with empty node pattern");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with invalid node pattern. Expects BreadCrumbException.
     */
    public void testConstructor_InvalidNodePattern() {
        try {
            new RegexNodeMatcher(TITLE, URL, "(ab");
            fail("Creates RegexNodeMatcher with invalid node pattern");
        } catch (BreadCrumbException e) {
            // good
        }
    }

    /**
     * Test of matches method with null node. Should return false.
     */
    public void testMatches_NullNode() {
        assertFalse("Returns true for null node", nodeMatcher.matches(null));
    }

    /**
     * Test of matches method with non-matched node. Should return false.
     */
    public void testMatches_NonMatchedNode() {
        assertFalse("Returns true for non-matched node", nodeMatcher.matches(new BreadCrumbNode("url", null)));
    }

    /**
     * Test of matches method with matched node. Should return true.
     */
    public void testMatches_MatchedNode() {
        assertTrue("Returns false for matched node", nodeMatcher.matches(new BreadCrumbNode("123", null)));
    }

    /**
     * Test of getNodePattern method. Verifies if it returns the node pattern set in constructor.
     */
    public void testGetNodePattern() {
        assertEquals("Returns an incorrect node pattern", NODE_PATTERN, nodeMatcher.getNodePattern());
    }

    /**
     * Test of equals method with null object. Should return false.
     */
    public void testEquals_NullObj() {
        assertFalse("Fails to compare equality for null object", nodeMatcher.equals(null));
    }

    /**
     * Test of equals method with object not of type RegexNodeMatcher. Should return false.
     */
    public void testEquals_WrongTypeObj() {
        assertFalse("Fails to compare equality for wrong type object", nodeMatcher.equals("string"));
    }

    /**
     * Test of equals method with RegexNodeMatcher object with different fields. Should return false.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testEquals_DiffObj() throws Exception {
        assertFalse("Fails to compare equality for different RegexNodeMatcher object",
            nodeMatcher.equals(new RegexNodeMatcher(TITLE, URL, "456")));
    }

    /**
     * Test of equals method with RegexNodeMatcher object with same fields. Should return true.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testEquals_SameObj() throws Exception {
        assertTrue("Fails to compare equality for same RegexNodeMatcher object",
            nodeMatcher.equals(new RegexNodeMatcher(TITLE, URL, NODE_PATTERN)));
    }

    /**
     * Test of hashCode method. Verifies if two equal RegexNodeMatcher objects return the same hash code.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testHashCode() throws Exception {
        RegexNodeMatcher nodeMatcher2 = new RegexNodeMatcher(TITLE, URL, NODE_PATTERN);

        assertEquals("Returns an incorrect hash code", nodeMatcher.hashCode(), nodeMatcher2.hashCode());
    }
}
